@@ -1,9 +1,9 @@
-// SPDX-FileCopyrightText: none
-// SPDX-License-Identifier: GPL-3.0-only
+// SPDX-FileCopyrightText: 2022 Plata Hill <plata.hill@kdemail.net>
+// SPDX-License-Identifier: LGPL-2.1-or-later
 
 #include "database.h"
 
-#include "fetcher.h"
+#include "TellySkoutSettings.h"
 
 #include <QDateTime>
 #include <QDebug>
@@ -28,7 +28,7 @@ Database::Database()
     }
 
     // drop DB if it doesn't use the correct fetcher
-    if (m_settings.fetcher() != fetcher()) {
+    if (TellySkoutSettings::fetcher() != fetcher()) {
         if (!dropTables()) {
             qCritical() << "Failed to drop database";
         }
@@ -107,7 +107,7 @@ Database::Database()
         qCritical() << "Failed to prepare database queries";
     }
 
-    connect(&m_settings, &TellySkoutSettings::fetcherChanged, this, [this]() {
+    connect(TellySkoutSettings::self(), &TellySkoutSettings::fetcherChanged, this, [this]() {
         dropTables();
         createTables();
     });
@@ -118,7 +118,7 @@ bool Database::createTables()
     qDebug() << "Create DB tables";
 
     TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Fetcher (id INTEGER UNIQUE);")));
-    TRUE_OR_RETURN(execute(QStringLiteral("INSERT OR IGNORE INTO Fetcher VALUES (") + QString::number(m_settings.fetcher()) + ");"));
+    TRUE_OR_RETURN(execute(QStringLiteral("INSERT OR IGNORE INTO Fetcher VALUES (") + QString::number(TellySkoutSettings::fetcher()) + ");"));
 
     TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS \"Groups\" (id TEXT UNIQUE, name TEXT, url TEXT);")));
     TRUE_OR_RETURN(execute(QStringLiteral("CREATE TABLE IF NOT EXISTS Channels (id TEXT UNIQUE, name TEXT, url TEXT, image TEXT);")));
@@ -223,34 +223,49 @@ int Database::fetcher() const
 
 void Database::cleanup()
 {
-    const unsigned int days = m_settings.deleteProgramAfter();
+    // delete programs in the past to avoid that the database grows over time
+    const unsigned int daysPast = TellySkoutSettings::deleteProgramAfter();
+    QDateTime dateTimePast = QDateTime::currentDateTime();
+    dateTimePast = dateTimePast.addDays(-static_cast<qint64>(daysPast));
 
-    QDateTime dateTime = QDateTime::currentDateTime();
-    dateTime = dateTime.addDays(-static_cast<qint64>(days));
-    const qint64 sinceEpoch = dateTime.toSecsSinceEpoch();
+    // delete programs in the far future (probably they have been added by mistake)
+    QDateTime dateTimeFuture = QDateTime::currentDateTime();
+    dateTimeFuture = dateTimeFuture.addDays(30);
 
     QSqlQuery query;
-    if (!query.prepare(QStringLiteral("DELETE FROM Programs WHERE stop < :sinceEpoch;"))) {
+    if (!query.prepare(QStringLiteral("DELETE FROM Programs WHERE stop < :sinceEpochPast OR stop > :sinceEpochFuture;"))) {
         qCritical() << "Failed to prepare cleanup query";
         return;
     }
-    query.bindValue(QStringLiteral(":sinceEpoch"), sinceEpoch);
+    query.bindValue(QStringLiteral(":sinceEpochPast"), dateTimePast.toSecsSinceEpoch());
+    query.bindValue(QStringLiteral(":sinceEpochFuture"), dateTimeFuture.toSecsSinceEpoch());
     execute(query);
 }
 
-void Database::addGroup(const GroupId &id, const QString &name, const QString &url)
+void Database::addGroup(const GroupData &data)
 {
-    if (!groupExists(id)) {
-        qDebug() << "Add group" << name;
-        m_addGroupQuery->bindValue(QStringLiteral(":id"), id.value());
-        m_addGroupQuery->bindValue(QStringLiteral(":name"), name);
-        m_addGroupQuery->bindValue(QStringLiteral(":url"), url);
+    if (!groupExists(data.m_id)) {
+        qDebug() << "Add group" << data.m_name;
+        m_addGroupQuery->bindValue(QStringLiteral(":id"), data.m_id.value());
+        m_addGroupQuery->bindValue(QStringLiteral(":name"), data.m_name);
+        m_addGroupQuery->bindValue(QStringLiteral(":url"), data.m_url);
         execute(*m_addGroupQuery);
 
-        Q_EMIT groupAdded(id);
+        Q_EMIT groupAdded(data.m_id);
     }
 }
 
+void Database::addGroups(const QVector<GroupData> &groups)
+{
+    QSqlDatabase::database().transaction();
+
+    for (const GroupData &data : groups) {
+        addGroup(data);
+    }
+
+    QSqlDatabase::database().commit();
+}
+
 size_t Database::groupCount() const
 {
     execute(*m_groupCountQuery);
@@ -328,6 +343,17 @@ void Database::addChannel(const ChannelData &data, const GroupId &group)
     }
 }
 
+void Database::addChannels(const QList<ChannelData> &channels, const GroupId &group)
+{
+    QSqlDatabase::database().transaction();
+
+    for (const ChannelData &data : channels) {
+        addChannel(data, group);
+    }
+
+    QSqlDatabase::database().commit();
+}
+
 size_t Database::channelCount() const
 {
     execute(*m_channelCountQuery);
@@ -507,18 +533,17 @@ void Database::addPrograms(const QVector<ProgramData> &programs)
 {
     QSqlDatabase::database().transaction();
 
-    for (int i = 0; i < programs.length(); i++) {
-        const ProgramData &data = programs.at(i);
+    for (const ProgramData &data : programs) {
         addProgram(data);
     }
 
     QSqlDatabase::database().commit();
 }
 
-bool Database::programExists(const ChannelId &channelId, qint64 lastTime) const
+bool Database::programExists(const ChannelId &channelId, const QDateTime &lastTime) const
 {
     m_programExistsQuery->bindValue(QStringLiteral(":channel"), channelId.value());
-    m_programExistsQuery->bindValue(QStringLiteral(":lastTime"), lastTime);
+    m_programExistsQuery->bindValue(QStringLiteral(":lastTime"), lastTime.toSecsSinceEpoch());
     execute(*m_programExistsQuery);
     m_programExistsQuery->next();
 
